/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

#include <cellular_modem.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <debug.h>

#define MAX_BUFF_LEN	1024

/****************************************
 * Function Name: do_return
 * action	: unlock file before return
 * Arg: file desc, return value
 * Return Value(return interger)
 *
*****************************************/
int do_return(unsigned int fd, int val)
{
	cellular_modem_close(fd) ;
	return val ;
}

/****************************************
 * Function Name: do_connection
 * action	: gprs connection
 * Arg:
 * 	argv[1]=<username>:<password>
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
int do_connection(char *auth)
{
	int fd ;
	char username[MAX_BUFF_LEN] ;
	char password[MAX_BUFF_LEN] ;
	char *p ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	/* initial data */
	memset(username,'\0',MAX_BUFF_LEN) ;
	memset(password,'\0',MAX_BUFF_LEN) ;
	p = strtok(auth,":") ;
	if(p) {
		strcpy(username,p) ;
		p = strtok(NULL,":") ;
		if(p)
		       	strcpy(password,p) ;
	}

	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	/* check if connection use authentication */
	if((username[0] != '\0') &&(password[0] != '\0')) {
		if(cellular_modem_gprs_establish_connection(fd,username,password)) {
			printf("error cellular_modem_gprs_establish_connection\n") ;
			return 3 ;
		}
	}
	else {
		if(cellular_modem_gprs_establish_connection(fd,NULL,NULL)) {
			printf("error cellular_modem_gprs_establish_connection\n") ;
			return 3 ;
		}
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_reconnection
 * action	: turn down gprs connection
 		  and do connect
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
int do_reconnection(void)
{
	int fd ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	if(cellular_modem_gprs_check_connection_status(fd)) {
	      cellular_modem_gprs_establish_connection(fd,NULL,NULL) ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_disconnection
 * action	: turn down gprs connection
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
int do_disconnection(void)
{
	int fd ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_gprs_abort_connection(fd)) {
		printf("error cellular_modem_gprs_abort_connection\n") ;
		return 2 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_connection_status
 * action	: query gprs connection
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: is connect=1
 *		: not connect=0
 *		: failure=negative
 *
*****************************************/
int do_connection_status(void)
{
	int fd ;

	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	if(!cellular_modem_gprs_check_connection_status(fd)) {
		printf("ppp0 is connect.\n") ;
	}
	else {
		printf("ppp0 is not connect.\n") ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_gprs_diagnose
 * action	: diagnose gprs modem,
 *		  pin code, baudrate,
 *		  temperature...
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int do_gprs_diagnose(void)
{
	int fd ;
	int val = 0 ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
	}
	val = cellular_modem_gprs_diagnose_status(fd) ;
	if(val&GPRS_ERROR_TEMPERATURE)
	       	printf("temperature error\n") ;
       	if(val&GPRS_ERROR_SIGNAL_STRENGTH)
	       	printf("signal error\n") ;
       	if(val&GPRS_ERROR_RADIOBAND)
	       	printf("radio band error\n") ;
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_pin_authentication
 * action	: use pin to authentication
 * Arg:
 * 	argv[1]=pin number(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int do_pin_authentication(char *pin)
{
	int fd ;

	PDEBUG("---------->%s,%d:pin%s\n",__FUNCTION__,__LINE__,pin) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sim_authenticate_pin_code(fd,pin) == 0)
		printf("password authentication success.\n") ;
	else {
		printf("password authentication failure.\n") ;
		return 1 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: do_sim_lock
 * action	: use pin to lock after 
 *		  every reset
 * Arg:
 * 	argv[1]=pin number(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int do_sim_lock(char *pin)
{
	int fd ;

	PDEBUG("---------->%s,%d:pin%s\n",__FUNCTION__,__LINE__,pin) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sim_enable_pin_code(fd,pin,1)!=0) {
		printf("set sim lock error.\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	else
		printf("set sim lock success.\n") ;
	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: do_sim_unlock
 * action	: disable input pin every
 *		  reset has done.
 * Arg:
 * 	argv[1]=pin number(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int do_sim_unlock(char *pin)
{
	int fd ;

	PDEBUG("---------->%s,%d:pin%s\n",__FUNCTION__,__LINE__,pin) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sim_enable_pin_code(fd,pin,0)!=0) {
		printf("set sim unlock error.\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	else
		printf("set sim unlock success.\n") ;
	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: renew_pin_number
 * action	: replace old pin with new pin
 * Arg:
 * 	argv[1]=pin number(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int renew_pin_number(char *pin)
{
	int fd ;
	char old_pin[MAX_BUFF_LEN] ;
	char new_pin[MAX_BUFF_LEN] ;
	char *p ;

	PDEBUG("---------->%s,%d:pin%s\n",__FUNCTION__,__LINE__,pin) ;
	/* initial data */
	memset(old_pin,'\0',MAX_BUFF_LEN) ;
	memset(new_pin,'\0',MAX_BUFF_LEN) ;
	p = strtok(pin,":") ;
	if(p) {
		strcpy(old_pin,p) ;
		p = strtok(NULL,":") ;
		if(p)
		       	strcpy(new_pin,p) ;
	}
	/* both arg must exit */
	if((old_pin[0] == '\0') || (new_pin[0] == '\0')) {
		return 2 ;
	}

	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}

	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}


	if(cellular_modem_sim_assign_pin_code(fd,old_pin,new_pin)==0) {
		printf("new pin code assign success\n") ;
	}
	else {
		printf("new pin code assign failure\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}

	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: do_puk_authentication
 * action	: use puk to authentication,
 *		  renew pin if success.
 * Arg:
 * 	argv[1]=puk number(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int do_puk_authentication(char *auth)
{
	int fd ;
	char puk[MAX_BUFF_LEN] ;
	char pin[MAX_BUFF_LEN] ;
	char *p ;

	PDEBUG("---------->%s,%d:puk%s\n",__FUNCTION__,__LINE__,puk) ;
	/* initial data */
	memset(puk,'\0',MAX_BUFF_LEN) ;
	memset(pin,'\0',MAX_BUFF_LEN) ;
	p = strtok(auth,":") ;
	if(p) {
		strcpy(puk,p) ;
		p = strtok(NULL,":") ;
		if(p)
		       	strcpy(pin,p) ;
	}

	/* both arg must exit */
	if((puk[0] == '\0') || (pin[0] == '\0')) {
		return 2 ;
	}
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sim_unlock_pin_code(fd,puk,pin)==0) {
		printf("puk authentication success\n") ;
	}
	else {
		printf("puk authentication failure\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: display_imsi
 * action	: show imsi information
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
#define AT_CIMI	"AT+CIMI"
int display_imsi(void)
{
	int fd ;
	char recv_data[MAX_BUFF_LEN] ;
	char *imsi ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return -1 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return -1 ;
	}
       	if(cellular_modem_send_cmd(fd, AT_CIMI, recv_data, MAX_BUFF_LEN, AT_MAX_TIMEOUT)>=0) {
	       	if(strtok(recv_data,"\r\n")) {
		       	imsi = strtok(NULL,"\r\n") ;
			if(imsi)
			       	printf("%s\n",imsi) ;
			else
				printf("fail to get imsi.\n") ;
		}
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: send_sms_by_text
 * action	: send text 
 * Arg:
 * 	argv[1]=text content(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int send_sms_by_text(char *str)
{
	int fd ;
	SMSMSG mysms ;
	char *p ;

	/* initial data */
	memset(&mysms,'\0',sizeof(char)*sizeof(SMSMSG)) ;
	p = strtok(str,":") ;
	if(p) {
		strcpy(mysms.phone_number,p) ;
		p = strtok(NULL,":") ;
		if(p) {
		       	strcpy(mysms.msg_text,p) ;
		       	p = strtok(NULL,":") ;
			if(p)
				mysms.msg_length = atoi(p) ;
		}
	}

	/* both arg must exit */
	if((mysms.phone_number[0] == '\0') || (mysms.msg_text[0] == '\0') || (mysms.msg_length <=0)) {
		printf("argument lack phone:%s, text:%s, length:%d\n",mysms.phone_number,mysms.msg_text,mysms.msg_length) ;
		return 2 ;
	}
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}

	/* send sms */
	if(cellular_modem_sms_send_message(fd,0,&mysms)!=0) {
		return 2 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: send_sms_by_file
 * action	: send file 
 * Arg:
 * 	argv[1]=file path(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int send_sms_by_file(char *file)
{
	int fd ;

	printf("This option does not support yet.\n") ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return -1 ;
	}
	/* check pin ready */
       	if(cellular_modem_sim_get_sim_card_status(fd) != 0) {
		printf("pin not ready\n") ;
		cellular_modem_close(fd) ;
		return -1 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: read_sms_text_mode
 * action	: read sms in text mode
 * Arg:
 * 	argv[1]=index(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int read_sms_text_mode(char *index)
{
	int fd ;
	SMSMSG mysms ;

	PDEBUG("---------->%s,%d:index=%s\n",__FUNCTION__,__LINE__,index) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sms_recv_message(fd,atoi(index),0,&mysms)==0) {
		printf("Read sms success\n") ;
		printf("phone:%s\n",mysms.phone_number) ;
		printf("text:%s\n",mysms.msg_text) ;
		printf("date:%s\n",mysms.msg_date) ;
		printf("length:%d\n",mysms.msg_length) ;
		printf("time:%s\n",mysms.msg_time) ;
		printf("status:%s\n",(mysms.been_read)?"READ":"UNREAD") ;
	}
	else {
		printf("Read sms failure\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: delete_sms
 * action	: delete sms in text mode
 * Arg:
 * 	argv[1]=index(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int delete_sms(char *index)
{
	int fd ;

	PDEBUG("---------->%s,%d:index=%s\n",__FUNCTION__,__LINE__,index) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	if(cellular_modem_sms_delete_message(fd,atoi(index))==0) {
		printf("delete sms success\n") ;
	}
	else {
		printf("delete sms failure\n") ;
		cellular_modem_close(fd) ;
		return 2 ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: send_at_command
 * action	: do at command
 * Arg:
 * 	argv[1]=at command(string)
 * Return Value(interger)
 * 		: success=1
 *		: failure=negative
 *
*****************************************/
int send_at_command(char *command)
{
	int fd ;
	char recv_data[MAX_BUFF_LEN*1024] ;

	PDEBUG("---------->%s,%d:index=%s\n",__FUNCTION__,__LINE__,index) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
       	if(cellular_modem_send_cmd(fd, command, recv_data, MAX_BUFF_LEN*1024, AT_MAX_TIMEOUT*5)>=0) {
		printf("%s",recv_data) ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}
/****************************************
 * Function Name: do_reset
 * action	: hardware reset gprs modem
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
int do_reset(void)
{
	int fd ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* reset */
       	modem_reset(fd) ;
	cellular_modem_close(fd) ;
	return 0 ;
}

/****************************************
 * Function Name: do_sim_status
 * action	: query sim card status
 * Arg:
 * 	argv[1]=void
 * Return Value(interger)
 * 		: success=0
 *		: failure=negative
 *
*****************************************/
int do_sim_status(void)
{
	int fd ;

	PDEBUG("---------->%s,%d\n",__FUNCTION__,__LINE__) ;
	fd = cellular_modem_open() ;
	if(!fd) {
		perror("cellular_modem_open:") ;
		return 2 ;
	}
	/* check sim status */
	switch(cellular_modem_sim_get_sim_card_status(fd)) {
		case 0:
			printf("SIM Card is ready!\n") ;
			break ;
		case 1:
			printf("No SIM Card detected.\n") ;
			break ;
		case 2:
			printf("Need PIN code authentication.\n") ;
			break ;
		case 3:
			printf("Need PUK code authentication.\n") ;
			break ;
		default:
			printf("unknow status\n") ;
			break ;
	}
	cellular_modem_close(fd) ;
	return 0 ;
}
